package desCrypt;

public class roundTrip {
	
	private byte[][] IP = new byte[8][8];
	private byte[][] IP_Inverse = new byte[8][8];
	private byte[][] expansion = new byte[8][6];
	private byte[] leftIP = new byte[32];
	private byte[] rightIP = new byte[32];
	
	private sBox s = new sBox();
	
	public roundTrip(byte[][] InputArray){
		/**
		 * set IP Array
		 * */
		int k=7, index=1;
		for(int j=0;j<8;j++){
			for(int i=0;i<8;i++){
				IP[j][i] = InputArray[k--][index];
			}
			index = (index+2)%9;
			k=7;
		}
		
		k=0;
		/**
		 * set left IP Array
		 * */
		for(int j=0;j<4;j++){
			for(int i=0;i<8;i++){
				leftIP[k++] = IP[j][i];
			}
		}

		/**
		 * set right IP Array
		 * */
		k=0;
		for(int j=4;j<8;j++){
			for(int i=0;i<8;i++){
				rightIP[k++] = IP[j][i];
			}
		}
	}

	/**
	 * function swap
	 * */
	public void swap(){
		byte[] temp = new byte[32];
		temp = leftIP;
		leftIP = rightIP;
		rightIP = temp;
	}
	
	/**
	 * set the Inverse Initial Permutation
	 * */
	public void setIP_Inverse(){
		byte[][] merge = new byte[8][8];
		int k=0,index=0;
		/**
		 * swap leftIP & rightIP
		 * */
		swap();
		
		/**
		 * merge leftIP & rightIP
		 * */
		for(int j=0;j<4;j++){
			for(int i=0;i<8;i++){
				merge[j][i] = leftIP[k++];  
			}
		}
		k=0;
		for(int j=4;j<8;j++){
			for(int i=0;i<8;i++){
				merge[j][i] = rightIP[k++];
			}
		}
		
		/**
		 * set IP-Inverse Array
		 * */
		k=7; index=4;
		for(int j=0;j<8;j++){
			for(int i=0;i<8;i++){
				IP_Inverse[i][j] = merge[index][k--];
			}
			index = (index+5)%9;
			k=7;
		}
	}
	
	/**
	 * set the Expansion Permutation 
	 * */
	public void setExpansion(){
		/**
		 * set Expansion Permutation
		 * */
		int index=1, k=0;
		for(int i=0;i<32;i++){
			expansion[k][index++] = rightIP[i];
			if(index==5){
				index=1;
				k++;
			}
		}
		for(int j=0;j<8;j++){
			expansion[j][0] = expansion[(j+7)%8][4];
			expansion[j][5] = expansion[(j+1)%8][1];
		}
	}
	
	/**
	 * next R 
	 * */
	public byte[][] setRi(byte[][] input){
		byte[][] temp = new byte[8][6];
		byte[][] output = new byte[8][4];
		byte[] afterRightIP = new byte[32];
		
		for(int j =0;j<8;j++){
			for(int i=0;i<6;i++){
				temp[j][i]=(byte) (expansion[j][i]^input[j][i]);
			}
		}
		///debug
		System.out.println("\nE^K");
		for(int j =0;j<8;j++){
			for(int i=0;i<6;i++){
				System.out.print(temp[j][i]);
			}
			System.out.print(" ");
		}
		
		/**
		 * sBox 
		 * */
		s.test(temp);
		
		/**
		 * permutation 
		 * */
		output = s.getPermutation();
		System.out.println("\n\nPermutation ");
		
		int k=0;
		for(int j =0;j<8;j++){
			for(int i =0;i<4;i++){
				System.out.print(output[j][i]);
				output[j][i]=(byte) (output[j][i]^leftIP[k]);
				afterRightIP[k++] = output[j][i];
			}
			System.out.print(" ");
		}
		System.out.println();
		setLeftIP(getRightIP());
		setRightIP(afterRightIP);
		return output;
	}
	
	/**
	 * binary to Hex(32bit) 2迭
	 * */
	public String getHex(byte[][] input){
		byte[] temp = new byte[32];
		String bin ="";
		String hex = "";
		
		int k=0,l=0;
		for(int j=0;j<32;j++){
			bin = bin+input[k][l];
			temp[j] = input[k][l++];
			if(l==4){
				k++;
				l=0;
			}
		}
		hex = Long.toHexString(Long.parseLong(bin,2));
		return hex;
	}

	/**
	 * binary to Hex(32bit) 1迭
	 * */
	public String getHex(byte[] input){
		String bin ="";
		String hex = "";
		
		for(int j=0;j<32;j++){
			bin = bin+input[j];
		}
		hex = Long.toHexString(Long.parseLong(bin,2));
		return hex;
	}

	/**
	 * binary to Hex(64bit) 2迭
	 * */
	public String getHex64(byte[][] input){
		String bin1 ="";
		String bin2 ="";
		String hex1 = "";
		String hex2 = "";
		int k=0,l=0;
		for(int j=0;j<32;j++){
			bin1 = bin1+input[k][l++];
			if(l==8){
				k++;
				l=0;
			}
		}
		for(int j=0;j<32;j++){
			bin2 = bin2+input[k][l++];
			if(l==8){
				k++;
				l=0;
			}
		}
		try{
			hex1 = Long.toHexString(Long.parseLong(bin1,2));
			hex2 = Long.toHexString(Long.parseLong(bin2,2));
		}catch(Exception e){
			System.err.println(e);
		}
		return hex1+hex2;
	}
	
	public void setLeftIP(byte[] leftIP){
		this.leftIP = leftIP;
	}

	public void setRightIP(byte[] rightIP){
		this.rightIP = rightIP;
	}
	
	public byte[][] getIP(){
		return IP;
	}

	public byte[][] getIP_Inverse(){
		return IP_Inverse;
	}
	
	public byte[][] getExpansion(){
		return expansion;
	}
	
	public byte[] getLeftIP(){
		return leftIP;
	}

	public byte[] getRightIP(){
		return rightIP;
	}
}
